@@ -144,6 +144,7 @@ from __future__ import division
 from contextlib import contextmanager
 import os
 import platform
+import psutil
 import random
 import re
 import shlex
@@ -154,9 +155,6 @@ import tempfile
 from threading import Event
 import time
 
-from rpyc import BgServingThread
-from rpyc.utils.factory import unix_connect
-
 from pwnlib import adb
 from pwnlib import atexit
 from pwnlib import elf
@@ -170,6 +168,7 @@ from pwnlib.context import context
 from pwnlib.log import getLogger
 from pwnlib.timeout import Timeout
 from pwnlib.util import misc
+from pwnlib.util import packing
 from pwnlib.util import proc
 
 log = getLogger(__name__)
@@ -320,13 +319,13 @@ def _gdbserver_port(gdbserver, ssh):
 
     if process_created.startswith(b'ERROR:'):
         raise ValueError(
-            'Failed to spawn process under gdbserver. gdbserver error message: %s' % process_created
+            'Failed to spawn process under gdbserver. gdbserver error message: %r' % process_created
         )
 
     try:
         gdbserver.pid   = int(process_created.split()[-1], 0)
     except ValueError:
-        log.error('gdbserver did not output its pid (maybe chmod +x?): %s', six.ensure_str(process_created))
+        log.error('gdbserver did not output its pid (maybe chmod +x?): %r', process_created)
 
     listening_on = b''
     while b'Listening' not in listening_on:
@@ -425,7 +424,7 @@ def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, api=
 
         Send a command to Bash
 
-        >>> io.sendline("echo hello")
+        >>> io.sendline(b"echo hello")
         >>> io.recvline()
         b'hello\n'
 
@@ -449,7 +448,7 @@ def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, api=
 
         Send a command to Bash
 
-        >>> io.sendline("echo hello")
+        >>> io.sendline(b"echo hello")
         >>> io.recvline()
         b'hello\n'
 
@@ -494,7 +493,7 @@ def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, api=
         Connect to the SSH server and start a process on the server
 
         >>> shell = ssh('travis', 'example.pwnme', password='demopass')
-        >>> io = gdb.debug(['bash'],
+        >>> io = gdb.debug(['whoami'],
         ...                 ssh = shell,
         ...                 gdbscript = '''
         ... break main
@@ -503,7 +502,7 @@ def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, api=
 
         Send a command to Bash
 
-        >>> io.sendline("echo hello")
+        >>> io.sendline(b"echo hello")
 
         Interact with the process
         >>> io.interactive() # doctest: +SKIP
@@ -583,7 +582,7 @@ def debug(args, gdbscript=None, exe=None, ssh=None, env=None, sysroot=None, api=
     garbage = gdbserver.recvline(timeout=1)
 
     # Some versions of gdbserver output an additional message
-    garbage2 = gdbserver.recvline_startswith(b"Remote debugging from host ", timeout=1)
+    garbage2 = gdbserver.recvline_startswith(b"Remote debugging from host ", timeout=2)
 
     return gdbserver
 
@@ -773,7 +772,7 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
         ... ''')
         >>> io.recvline()
         b'Hello from process debugger!\n'
-        >>> io.sendline('echo Hello from bash && exit')
+        >>> io.sendline(b'echo Hello from bash && exit')
         >>> io.recvall()
         b'Hello from bash\n'
 
@@ -803,7 +802,7 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
 
             Interact with the program in a regular way
 
-            >>> io.sendline('echo Hello from bash && exit')
+            >>> io.sendline(b'echo Hello from bash && exit')
 
             Observe the results
 
@@ -824,7 +823,7 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
         ... ''')
         >>> io.recvline()
         b'Hello from remote debugger!\n'
-        >>> io.sendline('echo Hello from bash && exit')
+        >>> io.sendline(b'echo Hello from bash && exit')
         >>> io.recvall()
         b'Hello from bash\n'
 
@@ -840,7 +839,7 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
         ... ''')
         >>> io.recvline(timeout=5)  # doctest: +SKIP
         b'Hello from ssh debugger!\n'
-        >>> io.sendline('This will be echoed back')
+        >>> io.sendline(b'This will be echoed back')
         >>> io.recvline()
         b'This will be echoed back\n'
         >>> io.close()
@@ -891,7 +890,7 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
 
         pids = list(pidof(target))
         if not pids:
-            log.error('No such process: %s' % target)
+            log.error('No such process: %s', target)
         pid = pids[0]
         log.info('Attaching to youngest process "%s" (PID = %d)' %
                  (target, pid))
@@ -914,12 +913,9 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
             cmd = ['sshpass', '-p', shell.password] + cmd
         if shell.keyfile:
             cmd += ['-i', shell.keyfile]
-        exefile = target.executable
-        cmd += ['gdb -q %s %s -x "%s"' % (exefile,
-                                       target.pid,
-                                       tmpfile)]
+        cmd += ['gdb', '-q', target.executable, target.pid, '-x', tmpfile]
 
-        misc.run_in_new_terminal(' '.join(cmd))
+        misc.run_in_new_terminal(cmd)
         return
 
     elif isinstance(target, tubes.sock.sock):
@@ -971,7 +967,7 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
     elif isinstance(target, elf.corefile.Corefile):
         pre += 'target core %s\n' % target.path
     else:
-        log.error("don't know how to attach to target: %r" % target)
+        log.error("don't know how to attach to target: %r", target)
 
     # if we have a pid but no exe, just look it up in /proc/
     if pid and not exe:
@@ -984,30 +980,29 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
         log.error('could not find target process')
 
     gdb_binary = binary()
-    cmd = gdb_binary
+    cmd = [gdb_binary]
 
     if gdb_args:
-        cmd += ' '
-        cmd += ' '.join(gdb_args)
+        cmd += gdb_args
 
     if context.gdbinit:
-        cmd += ' -nh '                     # ignore ~/.gdbinit
-        cmd += ' -x %s ' % context.gdbinit # load custom gdbinit
+        cmd += ['-nh']                  # ignore ~/.gdbinit
+        cmd += ['-x', context.gdbinit]  # load custom gdbinit
 
-    cmd += ' -q '
+    cmd += ['-q']
 
     if exe and context.native:
         if not ssh and not os.path.isfile(exe):
-            log.error('No such file: %s' % exe)
-        cmd += ' "%s"' % exe
+            log.error('No such file: %s', exe)
+        cmd += [exe]
 
     if pid and not context.os == 'android':
-        cmd += ' %d' % pid
+        cmd += [str(pid)]
 
     if context.os == 'android' and pid:
         runner  = _get_runner()
         which   = _get_which()
-        gdb_cmd = _gdbserver_args(pid=pid, which=which, env=env)
+        gdb_cmd = _gdbserver_args(pid=pid, which=which)
         gdbserver = runner(gdb_cmd)
         port    = _gdbserver_port(gdbserver, None)
         host    = context.adb_host
@@ -1033,14 +1028,14 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
     if gdbscript:
         tmp = tempfile.NamedTemporaryFile(prefix = 'pwn', suffix = '.gdb',
                                           delete = False, mode = 'w+')
-        log.debug('Wrote gdb script to %r\n%s' % (tmp.name, gdbscript))
+        log.debug('Wrote gdb script to %r\n%s', tmp.name, gdbscript)
         gdbscript = 'shell rm %s\n%s' % (tmp.name, gdbscript)
 
         tmp.write(gdbscript)
         tmp.close()
-        cmd += ' -x %s' % (tmp.name)
+        cmd += ['-x', tmp.name]
 
-    log.info('running in new terminal: %s' % cmd)
+    log.info('running in new terminal: %s', cmd)
 
     if api:
         # prevent gdb_faketerminal.py from messing up api doctests
@@ -1057,6 +1052,8 @@ def attach(target, gdbscript = '', exe = None, gdb_args = None, ssh = None, sysr
         return gdb_pid
 
     # connect to the GDB Python API bridge
+    from rpyc import BgServingThread
+    from rpyc.utils.factory import unix_connect
     if six.PY2:
         retriable = socket.error
     else:
@@ -1215,23 +1212,23 @@ def find_module_addresses(binary, ssh=None, ulimit=False):
 
     with runner(cmd) as gdb:
         if context.aslr:
-            gdb.sendline('set disable-randomization off')
+            gdb.sendline(b'set disable-randomization off')
 
-        gdb.send("""
+        gdb.send(b"""\
         set prompt
         catch load
         run
         """)
-        gdb.sendline('info sharedlibrary')
-        lines = context._decode(gdb.recvrepeat(2))
+        gdb.sendline(b'info sharedlibrary')
+        lines = packing._decode(gdb.recvrepeat(2))
 
         for line in lines.splitlines():
             m = expr.match(line)
             if m:
                 libs[m.group(2)] = int(m.group(1),16)
-        gdb.sendline('kill')
-        gdb.sendline('y')
-        gdb.sendline('quit')
+        gdb.sendline(b'kill')
+        gdb.sendline(b'y')
+        gdb.sendline(b'quit')
 
     #
     # Fix up all of the addresses against the .text address
@@ -1296,11 +1293,11 @@ def corefile(process):
     gdb_args = ['-batch',
                 '-q',
                 '-nx',
-                '-ex', '"set pagination off"',
-                '-ex', '"set height 0"',
-                '-ex', '"set width 0"',
-                '-ex', '"set use-coredump-filter on"',
-                '-ex', '"generate-core-file %s"' % corefile_path,
+                '-ex', 'set pagination off',
+                '-ex', 'set height 0',
+                '-ex', 'set width 0',
+                '-ex', 'set use-coredump-filter on',
+                '-ex', 'generate-core-file %s' % corefile_path,
                 '-ex', 'detach']
 
     with context.local(terminal = ['sh', '-c']):
@@ -1308,8 +1305,8 @@ def corefile(process):
             pid = attach(process, gdb_args=gdb_args)
             log.debug("Got GDB pid %d", pid)
             try:
-                os.waitpid(pid, 0)
-            except Exception:
+                psutil.Process(pid).wait()
+            except psutil.Error:
                 pass
 
     if not os.path.exists(corefile_path):
