@@ -541,8 +541,10 @@ def isprint(c):
     """isprint(c) -> bool
 
     Return True if a character is printable"""
-    t = (string.ascii_letters + string.digits + string.punctuation + ' ').encode()
-    return c in set(map(packing._p8lu, bytearray(t)))|set(map(chr, bytearray(t)))
+    if isinstance(c, six.text_type):
+        c = ord(c)
+    t = bytearray(string.ascii_letters + string.digits + string.punctuation + ' ', 'ascii')
+    return c in t
 
 
 def hexii(s, width = 16, skip = True):
@@ -800,8 +802,7 @@ def hexdump(s, width=16, skip=True, hexii=False, begin=0, style=None,
         dead0018  .g  .a  .a  .a   .h  .a  .a  .a  │
         dead0020
 
-        >>> import struct
-        >>> print(hexdump(list(map(struct.Struct("B").pack, range(256)))))
+        >>> print(hexdump(bytearray(range(256))))
         00000000  00 01 02 03  04 05 06 07  08 09 0a 0b  0c 0d 0e 0f  │····│····│····│····│
         00000010  10 11 12 13  14 15 16 17  18 19 1a 1b  1c 1d 1e 1f  │····│····│····│····│
         00000020  20 21 22 23  24 25 26 27  28 29 2a 2b  2c 2d 2e 2f  │ !"#│$%&'│()*+│,-./│
@@ -820,7 +821,7 @@ def hexdump(s, width=16, skip=True, hexii=False, begin=0, style=None,
         000000f0  f0 f1 f2 f3  f4 f5 f6 f7  f8 f9 fa fb  fc fd fe ff  │····│····│····│····│
         00000100
 
-        >>> print(hexdump(list(map(struct.Struct("B").pack, range(256))), hexii=True))
+        >>> print(hexdump(bytearray(range(256)), hexii=True))
         00000000      01  02  03   04  05  06  07   08  09  0a  0b   0c  0d  0e  0f  │
         00000010  10  11  12  13   14  15  16  17   18  19  1a  1b   1c  1d  1e  1f  │
         00000020  20  .!  ."  .#   .$  .%  .&  .'   .(  .)  .*  .+   .,  .-  ..  ./  │
@@ -934,14 +935,14 @@ def hexdump(s, width=16, skip=True, hexii=False, begin=0, style=None,
         00000010  41 41 41 41 41 41 41 41                          │AAAAAAAA│
         00000018
 
-        >>> print(hexdump('A'*24, width=16, total=False))
+        >>> print(hexdump(b'A'*24, width=16, total=False))
         00000000  41 41 41 41  41 41 41 41  41 41 41 41  41 41 41 41  │AAAA│AAAA│AAAA│AAAA│
         00000010  41 41 41 41  41 41 41 41                            │AAAA│AAAA│
-        >>> print(hexdump('A'*24, width=16, groupsize=8, total=False))
+        >>> print(hexdump(b'A'*24, width=16, groupsize=8, total=False))
         00000000  41 41 41 41 41 41 41 41  41 41 41 41 41 41 41 41  │AAAAAAAA│AAAAAAAA│
         00000010  41 41 41 41 41 41 41 41                           │AAAAAAAA│
     """
-    s = packing.flat(s)
+    s = packing.flat(s, stacklevel=1)
     return '\n'.join(hexdump_iter(BytesIO(s),
                                   width,
                                   skip,
