@@ -37,6 +37,7 @@ import collections
 import six
 import struct
 import sys
+import warnings
 
 from six.moves import range
 
@@ -50,7 +51,7 @@ mod = sys.modules[__name__]
 log = getLogger(__name__)
 
 def pack(number, word_size = None, endianness = None, sign = None, **kwargs):
-    """pack(number, word_size = None, endianness = None, sign = None, **kwargs) -> str
+    r"""pack(number, word_size = None, endianness = None, sign = None, **kwargs) -> str
 
     Packs arbitrary-sized integer.
 
@@ -80,23 +81,23 @@ def pack(number, word_size = None, endianness = None, sign = None, **kwargs):
         >>> pack(0x414243, 24, 'little', True)
         b'CBA'
         >>> pack(0x814243, 24, 'big', False)
-        b'\\x81BC'
+        b'\x81BC'
         >>> pack(0x814243, 24, 'big', True)
         Traceback (most recent call last):
            ...
         ValueError: pack(): number does not fit within word_size
         >>> pack(0x814243, 25, 'big', True)
-        b'\\x00\\x81BC'
+        b'\x00\x81BC'
         >>> pack(-1, 'all', 'little', True)
-        b'\\xff'
+        b'\xff'
         >>> pack(-256, 'all', 'big', True)
-        b'\\xff\\x00'
+        b'\xff\x00'
         >>> pack(0x0102030405, 'all', 'little', True)
-        b'\\x05\\x04\\x03\\x02\\x01'
+        b'\x05\x04\x03\x02\x01'
         >>> pack(-1)
-        b'\\xff\\xff\\xff\\xff'
+        b'\xff\xff\xff\xff'
         >>> pack(0x80000000, 'all', 'big', True)
-        b'\\x00\\x80\\x00\\x00\\x00'
+        b'\x00\x80\x00\x00\x00'
 """
     if sign is None and number < 0:
         sign = True
@@ -165,7 +166,7 @@ def pack(number, word_size = None, endianness = None, sign = None, **kwargs):
 
 @LocalNoarchContext
 def unpack(data, word_size = None):
-    """unpack(data, word_size = None, endianness = None, sign = None, **kwargs) -> int
+    r"""unpack(data, word_size = None, endianness = None, sign = None, **kwargs) -> int
 
     Packs arbitrary-sized integer.
 
@@ -188,17 +189,17 @@ def unpack(data, word_size = None):
         The unpacked number.
 
     Examples:
-        >>> hex(unpack(b'\\xaa\\x55', 16, endian='little', sign=False))
+        >>> hex(unpack(b'\xaa\x55', 16, endian='little', sign=False))
         '0x55aa'
-        >>> hex(unpack(b'\\xaa\\x55', 16, endian='big', sign=False))
+        >>> hex(unpack(b'\xaa\x55', 16, endian='big', sign=False))
         '0xaa55'
-        >>> hex(unpack(b'\\xaa\\x55', 16, endian='big', sign=True))
+        >>> hex(unpack(b'\xaa\x55', 16, endian='big', sign=True))
         '-0x55ab'
-        >>> hex(unpack(b'\\xaa\\x55', 15, endian='big', sign=True))
+        >>> hex(unpack(b'\xaa\x55', 15, endian='big', sign=True))
         '0x2a55'
-        >>> hex(unpack(b'\\xff\\x02\\x03', 'all', endian='little', sign=True))
+        >>> hex(unpack(b'\xff\x02\x03', 'all', endian='little', sign=True))
         '0x302ff'
-        >>> hex(unpack(b'\\xff\\x02\\x03', 'all', endian='big', sign=True))
+        >>> hex(unpack(b'\xff\x02\x03', 'all', endian='big', sign=True))
         '-0xfdfd'
     """
 
@@ -206,6 +207,7 @@ def unpack(data, word_size = None):
     word_size  = word_size  or context.word_size
     endianness = context.endianness
     sign       = context.sign
+    data = _need_bytes(data, 2)
 
     # Verify that word_size make sense
     if word_size == 'all':
@@ -289,13 +291,19 @@ def unpack_many(data, word_size = None):
 #
 # Make individual packers, e.g. _p8lu
 #
-ops   = {'p': struct.pack, 'u': lambda *a: struct.unpack(*(
-                                             x.encode('latin1') if not hasattr(x, 'decode') else x
-                                             for x in a))[0]}
+ops   = ['p','u']
 sizes = {8:'b', 16:'h', 32:'i', 64:'q'}
 ends  = ['b','l']
 signs = ['s','u']
 
+return_types     = {'p': 'bytes', 'u': 'int'}
+op_verbs         = {'p': 'pack', 'u': 'unpack'}
+arg_doc          = {'p': 'number (int): Number to convert',
+                    'u': 'data (bytes): Byte string to convert'}
+rv_doc           = {'p': 'The packed number as a byte string',
+                    'u': 'The unpacked number'}
+
+
 def make_single(op,size,end,sign):
     name = '_%s%s%s%s' % (op, size, end, sign)
     fmt  = sizes[size]
@@ -305,22 +313,23 @@ def make_single(op,size,end,sign):
         fmt = fmt.upper()
     fmt = end+fmt
 
-    def routine(data):
-        return ops[op](fmt,data)
+    struct_op = getattr(struct.Struct(fmt), op_verbs[op])
+    if op == 'u':
+        def routine(data, stacklevel=1):
+            data = _need_bytes(data, stacklevel)
+            return struct_op(data)[0]
+    else:
+        def routine(data, stacklevel=None):
+            return struct_op(data)
     routine.__name__ = routine.__qualname__ = name
 
     return name, routine
 
+
 for op,size,end,sign in iters.product(ops, sizes, ends, signs):
     name, routine = make_single(op,size,end,sign)
     setattr(mod, name, routine)
 
-return_types     = {'p': 'str', 'u': 'int'}
-op_verbs         = {'p': 'pack', 'u': 'unpack'}
-arg_doc          = {'p': 'number (int): Number to convert',
-                    'u': 'data (str): String to convert'}
-rv_doc           = {'p': 'The packed number as a string',
-                    'u': 'The unpacked number'}
 
 #
 # Make normal user-oriented packers, e.g. p8
@@ -341,7 +350,7 @@ def make_multi(op, size):
         return {("little", True  ): ls,
                 ("little", False):  lu,
                 ("big",    True  ): bs,
-                ("big",    False):  bu}[endian, signed](number)
+                ("big",    False):  bu}[endian, signed](number, 3)
 
     routine.__name__ = name
     routine.__doc__  = """%s%s(number, sign, endian, ...) -> %s
@@ -421,7 +430,7 @@ def make_packer(word_size = None, sign = None, **kwargs):
                 (64, 0, 1): _p64ls,
                 (64, 1, 0): _p64bu,
                 (64, 1, 1): _p64bs,
-            }.get((word_size, {'big': 1, 'little': 0}[endianness], sign), None)
+            }.get((word_size, {'big': 1, 'little': 0}[endianness], sign))
 
             if packer:
                 return packer
@@ -452,9 +461,9 @@ def make_unpacker(word_size = None, endianness = None, sign = None, **kwargs):
         >>> u = make_unpacker(32, endian='little', sign='unsigned')
         >>> u
         <function _u32lu at 0x...>
-        >>> hex(u('/bin'))
+        >>> hex(u(b'/bin'))
         '0x6e69622f'
-        >>> u('abcde')
+        >>> u(b'abcde')
         Traceback (most recent call last):
             ...
         error: unpack requires a string argument of length 4
@@ -489,7 +498,7 @@ def make_unpacker(word_size = None, endianness = None, sign = None, **kwargs):
     else:
         return lambda number: unpack(number, word_size, endianness, sign)
 
-def _fit(pieces, preprocessor, packer, filler):
+def _fit(pieces, preprocessor, packer, filler, stacklevel=1):
 
     # Pulls bytes from `filler` and adds them to `pad` until it ends in `key`.
     # Returns the index of `key` in `pad`.
@@ -511,10 +520,8 @@ def _fit(pieces, preprocessor, packer, filler):
         if isinstance(k, six.integer_types):
             if k >= large_key:
                 k = fill(pack(k))
-        elif isinstance(k, six.text_type):
-            k = fill(k.encode('utf8'))
-        elif isinstance(k, (bytearray, bytes)):
-            k = fill(k)
+        elif isinstance(k, (six.text_type, bytearray, bytes)):
+            k = fill(_encode(k))
         else:
             raise TypeError("flat(): offset must be of type int or str, but got '%s'" % type(k))
         if k in pieces_:
@@ -545,7 +552,7 @@ def _fit(pieces, preprocessor, packer, filler):
             out += p8(next(filler))
 
         # Recursively flatten data
-        out += _flat([v], preprocessor, packer, filler)
+        out += _flat([v], preprocessor, packer, filler, stacklevel + 1)
 
     # Now do negative indices
     out_negative = b''
@@ -562,11 +569,11 @@ def _fit(pieces, preprocessor, packer, filler):
                 out_negative += p8(next(filler))
 
             # Recursively flatten data
-            out_negative += _flat([v], preprocessor, packer, filler)
+            out_negative += _flat([v], preprocessor, packer, filler, stacklevel + 1)
 
     return filler, out_negative + out
 
-def _flat(args, preprocessor, packer, filler):
+def _flat(args, preprocessor, packer, filler, stacklevel=1):
     out = []
     for arg in args:
 
@@ -578,13 +585,13 @@ def _flat(args, preprocessor, packer, filler):
         if hasattr(arg, '__flat__'):
             val = arg.__flat__()
         elif isinstance(arg, (list, tuple)):
-            val = _flat(arg, preprocessor, packer, filler)
+            val = _flat(arg, preprocessor, packer, filler, stacklevel + 1)
         elif isinstance(arg, dict):
-            filler, val = _fit(arg, preprocessor, packer, filler)
+            filler, val = _fit(arg, preprocessor, packer, filler, stacklevel + 1)
         elif isinstance(arg, bytes):
             val = arg
         elif isinstance(arg, six.text_type):
-            val = arg.encode('utf8')
+            val = _need_bytes(arg, stacklevel + 1)
         elif isinstance(arg, six.integer_types):
             val = packer(arg)
         elif isinstance(arg, bytearray):
@@ -662,13 +669,13 @@ def flat(*args, **kwargs):
         b'X'
         >>> flat([1,2,3])
         b'\x01\x00\x00\x00\x02\x00\x00\x00\x03\x00\x00\x00'
-        >>> flat({4:'X'})
+        >>> flat({4:b'X'})
         b'aaaaX'
 
         :meth:`.flat` flattens all of the values provided, and allows nested lists
         and dictionaries.
 
-        >>> flat([{4:'X'}] * 2)
+        >>> flat([{4:b'X'}] * 2)
         b'aaaaXaaacX'
         >>> flat([[[[[[[[[1]]]], 2]]]]])
         b'\x01\x00\x00\x00\x02\x00\x00\x00'
@@ -676,20 +683,20 @@ def flat(*args, **kwargs):
         You can also provide additional arguments like endianness, word-size, and
         whether the values are treated as signed or not.
 
-        >>> flat(1, "test", [[["AB"]*2]*3], endianness = 'little', word_size = 16, sign = False)
+        >>> flat(1, b"test", [[[b"AB"]*2]*3], endianness = 'little', word_size = 16, sign = False)
         b'\x01\x00testABABABABABAB'
 
         A preprocessor function can be provided in order to modify the values in-flight.
-        This example converts increments each value by 1, then converts to a string.
+        This example converts increments each value by 1, then converts to a byte string.
 
-        >>> flat([1, [2, 3]], preprocessor = lambda x: str(x+1))
+        >>> flat([1, [2, 3]], preprocessor = lambda x: str(x+1).encode())
         b'234'
 
         Using dictionaries is a fast way to get specific values at specific offsets,
         without having to do ``data += "foo"`` repeatedly.
 
         >>> flat({12: 0x41414141,
-        ...       24: 'Hello',
+        ...       24: b'Hello',
         ...      })
         b'aaaabaaacaaaAAAAeaaafaaaHello'
 
@@ -699,32 +706,32 @@ def flat(*args, **kwargs):
 
         The cyclic pattern can be provided as either the text or hexadecimal offset.
 
-        >>> flat({ 0x61616162: 'X'})
+        >>> flat({ 0x61616162: b'X'})
         b'aaaaX'
-        >>> flat({'baaa': 'X'})
+        >>> flat({'baaa': b'X'})
         b'aaaaX'
 
         Fields do not have to be in linear order, and can be freely mixed.
         This also works with cyclic offsets.
 
-        >>> flat({2: 'A', 0:'B'})
+        >>> flat({2: b'A', 0:b'B'})
         b'BaA'
-        >>> flat({0x61616161:'x', 0x61616162:'y'})
+        >>> flat({0x61616161: b'x', 0x61616162: b'y'})
         b'xaaay'
-        >>> flat({0x61616162:'y', 0x61616161:'x'})
+        >>> flat({0x61616162: b'y', 0x61616161: b'x'})
         b'xaaay'
 
         Fields do not have to be in order, and can be freely mixed.
 
-        >>> flat({'caaa': 'XXXX', 16: '\x41', 20: 0xdeadbeef})
+        >>> flat({'caaa': b'XXXX', 16: b'\x41', 20: 0xdeadbeef})
         b'aaaabaaaXXXXdaaaAaaa\xef\xbe\xad\xde'
-        >>> flat({ 8: [0x41414141, 0x42424242], 20: 'CCCC'})
+        >>> flat({ 8: [0x41414141, 0x42424242], 20: b'CCCC'})
         b'aaaabaaaAAAABBBBeaaaCCCC'
         >>> fit({
-        ...     0x61616161: 'a',
-        ...     1: 'b',
-        ...     0x61616161+2: 'c',
-        ...     3: 'd',
+        ...     0x61616161: b'a',
+        ...     1: b'b',
+        ...     0x61616161+2: b'c',
+        ...     3: b'd',
         ... })
         b'abadbaaac'
 
@@ -732,22 +739,22 @@ def flat(*args, **kwargs):
         You can customize this by providing an iterable or method for the ``filler``
         argument.
 
-        >>> flat({12: 'XXXX'}, filler = b'_', length = 20)
+        >>> flat({12: b'XXXX'}, filler = b'_', length = 20)
         b'____________XXXX____'
-        >>> flat({12: 'XXXX'}, filler = b'AB', length = 20)
+        >>> flat({12: b'XXXX'}, filler = b'AB', length = 20)
         b'ABABABABABABXXXXABAB'
 
         Nested dictionaries also work as expected.
 
-        >>> flat({4: {0: 'X', 4: 'Y'}})
+        >>> flat({4: {0: b'X', 4: b'Y'}})
         b'aaaaXaaaY'
-        >>> fit({4: {4: 'XXXX'}})
+        >>> fit({4: {4: b'XXXX'}})
         b'aaaabaaaXXXX'
 
         Negative indices are also supported, though this only works for integer
         keys.
     
-        >>> flat({-4: 'x', -1: 'A', 0: '0', 4:'y'})
+        >>> flat({-4: b'x', -1: b'A', 0: b'0', 4: b'y'})
         b'xaaA0aaay'
     """
     # HACK: To avoid circular imports we need to delay the import of `cyclic`
@@ -756,15 +763,16 @@ def flat(*args, **kwargs):
     preprocessor = kwargs.pop('preprocessor', lambda x: None)
     filler       = kwargs.pop('filler', cyclic.de_bruijn())
     length       = kwargs.pop('length', None)
+    stacklevel   = kwargs.pop('stacklevel', 0)
 
-    if isinstance(filler, str):
-        filler = bytearray(six.ensure_binary(filler))
+    if isinstance(filler, (str, six.text_type)):
+        filler = bytearray(_need_bytes(filler))
 
     if kwargs != {}:
         raise TypeError("flat() does not support argument %r" % kwargs.popitem()[0])
 
     filler = iters.cycle(filler)
-    out = _flat(args, preprocessor, make_packer(), filler)
+    out = _flat(args, preprocessor, make_packer(), filler, stacklevel + 2)
 
     if length:
         if len(out) > length:
@@ -775,6 +783,7 @@ def flat(*args, **kwargs):
 
 def fit(*args, **kwargs):
     """Legacy alias for :func:`flat`"""
+    kwargs['stacklevel'] = kwargs.get('stacklevel', 0) + 1
     return flat(*args, **kwargs)
 
 """
@@ -1000,5 +1009,65 @@ def dd(dst, src, count = 0, skip = 0, seek = 0, truncate = False):
 
     return dst
 
+def _need_bytes(s, level=1):
+    if isinstance(s, (bytes, bytearray)):
+        return s   # already bytes
+
+    encoding = context.encoding
+    errors = 'strict'
+    if encoding == 'auto':
+        worst = max(map(ord, s))
+        if worst > 255:
+            encoding = 'UTF-8'
+            errors = 'surrogateescape'
+        elif worst > 127:
+            encoding = 'ISO-8859-1'
+        else:
+            encoding = 'ASCII'
+
+    warnings.warn("Text is not bytes; assuming {}, no guarantees. See https://docs.pwntools.com/#bytes"
+                  .format(encoding), BytesWarning, level + 2)
+    return s.encode(encoding, errors)
+
+def _need_text(s, level=1):
+    if isinstance(s, (str, six.text_type)):
+        return s   # already text
+
+    encoding = context.encoding
+    errors = 'strict'
+    if encoding == 'auto':
+        for encoding in 'ASCII', 'UTF-8', 'ISO-8859-1':
+            try:
+                s = s.decode(encoding)
+            except UnicodeDecodeError:
+                pass
+            else:
+                break
+
+    warnings.warn("Bytes is not text; assuming {}, no guarantees. See https://docs.pwntools.com/#bytes"
+                  .format(encoding), BytesWarning, level + 2)
+    return s.decode(encoding, errors)
+
+def _encode(s):
+    if isinstance(s, (bytes, bytearray)):
+        return s   # already bytes
+
+    if context.encoding == 'auto':
+        try:
+            return s.encode('latin1')
+        except UnicodeEncodeError:
+            return s.encode('utf-8', 'surrogateescape')
+    return s.encode(context.encoding)
+
+def _decode(b):
+    if context.encoding == 'auto':
+        try:
+            return b.decode('utf-8')
+        except UnicodeDecodeError:
+            return b.decode('latin1')
+        except AttributeError:
+            return b
+    return b.decode(context.encoding)
+
 del op, size, end, sign
 del name, routine, mod
